<?php
/**
 * This file is part of a NewQuest Project
 *
 * (c) NewQuest <contact@newquest.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * @author    NewQuest
 * @copyright NewQuest
 * @license   NewQuest
 */

require_once dirname(__FILE__).'/classes/NQLogger.php';
require_once dirname(__FILE__).'/classes/OzonClientApi.php';
require_once dirname(__FILE__).'/classes/OzonOrderIndex.php';
require_once dirname(__FILE__).'/classes/OzonUuidIndex.php';
require_once dirname(__FILE__).'/classes/OzonOrderTag.php';

class Ozon extends Module
{
    public function __construct()
    {
        $this->name = 'ozon';
        $this->tab = 'payment_security';
        $this->version = '1.0.7';
        $this->author = 'NewQuest';

        if (version_compare(_PS_VERSION_, '1.6', '>=')) {
            $this->bootstrap = true;
            $this->module_key = '6fd81fd02c5e9725bd9c00d76b7b9528';
        }

        parent::__construct();

        $this->displayName = 'OZON';
        $this->description = 'Protection contre les attaques & prévention des fraudes';
        $this->confirmUninstall = $this->l('Do you really want to uninstall this module ?');
    }

    /**
     * Install method.
     * @return bool
     */
    public function install()
    {
        if (!parent::install()
            || !$this->registerHook('header')
            || !$this->registerHook('displayAdminOrder')
            || !$this->registerHook('displayShoppingCart')
            || !$this->registerHook('displayBackOfficeHeader')
            || !$this->registerHook('actionValidateOrder')
        ) {
            return false;
        }

        // Create database table.
        include_once(dirname(__FILE__).'/sql/install.php');
        NQLogger::createTable();

        $this->createConfigVars();

        $this->createAdminTab();

        return true;
    }

    /**
     * Uninstall method.
     * @return bool
     */
    public function uninstall()
    {
        if (!parent::uninstall() || !Configuration::deleteByName(Tools::strtoupper($this->name.'_conf'))) {
            return false;
        }

        // Delete database table. (uncomment if you need to uninstall database table)
        include_once(dirname(__FILE__).'/sql/uninstall.php');
        NQLogger::dropTable();

        $this->deleteAdminTab();

        return true;
    }

    /**
     * Create config vars on install.
     */
    public function createConfigVars()
    {
        Configuration::updateValue('OZON_ACTIVE', 0);
        Configuration::updateValue('OZON_API_URL', 'https://api.ozon.io/v1/');
        Configuration::updateValue('OZON_WEBAPP_TOKEN', '');
        Configuration::updateValue('OZON_API_TOKEN', '');
        Configuration::updateValue('OZON_API_TOKEN_SECRET', '');
        Configuration::updateValue('OZON_CRON_TOKEN', md5(Tools::passwdGen(20)));
    }

    /**
     * Delete config vars on uninstall.
     */
    public function deleteConfigVars()
    {
        Configuration::deleteByName('OZON_ACTIVE');
        Configuration::deleteByName('OZON_API_URL');
        Configuration::deleteByName('OZON_WEBAPP_TOKEN');
        Configuration::deleteByName('OZON_API_TOKEN');
        Configuration::deleteByName('OZON_API_TOKEN_SECRET');
        Configuration::deleteByName('OZON_CRON_TOKEN');
    }

    /**
     * Create specific AdminTab on install.
     */
    private function createAdminTab()
    {
        $languages = Language::getLanguages(true);

        $tab_parent = new Tab();
        $tab_parent->active = 1;
        $tab_parent->class_name = 'AdminOzon';
        $tab_parent->name = array();
        foreach ($languages as $lang) {
            $tab_parent->name[$lang['id_lang']] = $this->displayName;
        }
        $tab_parent->id_parent = 0;
        $tab_parent->module = $this->name;
        if ($tab_parent->add()) {

            // Create order listing tab.
            $tab = new Tab();
            $tab->active = 1;
            $tab->class_name = 'AdminOzonOrders';
            $tab->name = array();
            foreach ($languages as $lang) {
                $tab->name[$lang['id_lang']] = 'Commandes';
            }
            $tab->id_parent = (int)$tab_parent->id;
            $tab->module = $this->name;
            $tab->add();

            // Create log listing tab.
            $tab = new Tab();
            $tab->active = 1;
            $tab->class_name = 'AdminNqLog';
            $tab->name = array();
            foreach ($languages as $lang) {
                $tab->name[$lang['id_lang']] = 'Logs';
            }
            $tab->id_parent = (int)$tab_parent->id;
            $tab->module = $this->name;
            $tab->add();
        }
    }

    /**
     * Delete specific AdminTab on uninstall.
     */
    private function deleteAdminTab()
    {
        $tab_to_delete = array(
            'AdminOzonOrders',
            'AdminNqLog',
            'AdminOzon',
        );

        foreach ($tab_to_delete as $tab_class_name) {
            $id_tab = (int)Tab::getIdFromClassName($tab_class_name);
            $tab = new Tab($id_tab);
            if (Validate::isLoadedObject($tab)) {
                $tab->delete();
            }
        }
    }

    //=====================================
    //  HOOK FUNCTIONS
    //=====================================
    /**
     * Check if module is active. (check module config var)
     * @return bool
     */
    private function moduleIsActive()
    {
        if (!Configuration::get('OZON_ACTIVE')) {
            return false;
        }

        return true;
    }


    /**
     * Display ozon order check results on admin order panel.
     * @param $params
     * @return mixed
     */
    public function hookDisplayAdminOrder($params)
    {
        $current_order = $this->context->link->getAdminLink('AdminOrders').'&id_order='.$params['id_order'].
            '&vieworder';

        if (Tools::getValue('ozon_reload')) {
            OzonOrderIndex::deleteIndex($params['id_order']);
            Tools::redirectAdmin($current_order);
        }

        $order_data = OzonOrderIndex::getIndex((int)$params['id_order']);
        if ($order_data) {

            $order_data['color_risk'] = $this->getColorOrderRisk($order_data['order_risk']);

            $id_cart = (int)CartCore::getCartIdByOrderId($order_data['id_order']);
            $order_data['ozon_uuid'] = OzonUuidIndex::getIndex($id_cart);

            if ($order_data['tags']) {
                $order_data['tags_list'] = OzonOrderTag::getTags(explode(':', $order_data['tags']));
            }
        }

        $this->context->smarty->assign(array(
            'order_data' => $order_data,
            'current_order' => $current_order,
        ));

        $template = 'adminorder.tpl';
        if (version_compare(_PS_VERSION_, '1.6', '<')) {
            $template = 'adminorder_15.tpl';
        }

        return $this->display(__FILE__, $template);
    }

    /**
     * Save current ozon_uuid for this cart to use on check api call.
	 *
     * @param $params
     */
    public function hookDisplayShoppingCart($params)
    {
        if (ValidateCore::isLoadedObject($params['cart']) && !empty($_COOKIE['ozon_uuid'])) {
            OzonUuidIndex::saveIndex((int)$params['cart']->id, $_COOKIE['ozon_uuid']);
        }
    }

	/**
	 * Hook back office header, add style
	 */
    public function hookDisplayBackOfficeHeader()
    {
        $this->context->controller->addCss($this->_path.'views/css/tab.css');
    }

	/**
	 * Hook Header
	 *
	 * @return mixed
	 */
    public function hookDisplayHeader()
    {
		return $this->display(__FILE__, 'header.tpl');
    }

    /**
     * Submit new validate order to OZON api.
     * @param $params
     * @return bool
     */
    public function hookActionValidateOrder($params)
    {
        $this->sendOrderToAPI($params['order']);
    }

    //=====================================
    //  ADMIN FUNCTIONS
    //=====================================
    /**
     * @return string
     */
    public function getContent()
    {
        $output = null;

        if (Tools::isSubmit('submit'.$this->name) && Tools::isSubmit('submitKey')) {

            $values = array();
            $values['OZON_ACTIVE'] = htmlentities(Tools::getValue('OZON_ACTIVE'));
            $values['OZON_API_URL'] = htmlentities(Tools::getValue('OZON_API_URL'));
            $values['OZON_WEBAPP_TOKEN'] = htmlentities(Tools::getValue('OZON_WEBAPP_TOKEN'));
            $values['OZON_API_TOKEN'] = htmlentities(Tools::getValue('OZON_API_TOKEN'));
            $values['OZON_API_TOKEN_SECRET'] = htmlentities(Tools::getValue('OZON_API_TOKEN_SECRET'));

            foreach ($values as $name => $value) {
                Configuration::updateValue($name, $value);
            }
        }

        $output .= $this->displayInformation();

        return $output.$this->renderForm();
    }

    /**
     * display information block on module top config form.
     * @return mixed
     */
    private function displayInformation()
    {
        $template = 'information.tpl';
        if (version_compare(_PS_VERSION_, '1.6', '<')) {
            $template = 'information_15.tpl';
        }

        return $this->display(__FILE__, $template);
    }

    /**
     * @return mixed
     */
    public function renderForm()
    {
        $fields_form = array();

        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Settings'),
            ),
            'input' => array(
                array(
                    'type' => 'hidden',
                    'name' => 'submitKey'
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save')
            )
        );

        if (version_compare(_PS_VERSION_, '1.6', '>=')) {
            $fields_form[0]['form']['input'][] = array(
                'type' => 'switch',
                'label' => $this->l('Script enable'),
                'name' => 'OZON_ACTIVE',
                'values' => array(
                    array(
                        'id' => 'active_on',
                        'value' => 1,
                        'label' => $this->l('Enabled')
                    ),
                    array(
                        'id' => 'active_off',
                        'value' => 0,
                        'label' => $this->l('Disabled')
                    )
                ),
            );
        } else {
            $fields_form[0]['form']['input'][] = array(
                'type' => 'radio',
                'label' => $this->l('Script enable'),
                'name' => 'OZON_ACTIVE',
                'class' => 't',
                'is_bool' => true,
                'values' => array(
                    array(
                        'id' => 'active_on',
                        'value' => 1,
                        'label' => $this->l('Enabled')
                    ),
                    array(
                        'id' => 'active_off',
                        'value' => 0,
                        'label' => $this->l('Disabled')
                    )
                )
            );
        }

        $fields_form[0]['form']['input'] = array_merge(
            $fields_form[0]['form']['input'],
            array(
                array(
                    'type' => 'text',
                    'label' => $this->l('Api url'),
                    'name' => 'OZON_API_URL',
                    'size' => 128,
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Token Webapp'),
                    'name' => 'OZON_WEBAPP_TOKEN',
                    'size' => 128,
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Token API'),
                    'name' => 'OZON_API_TOKEN',
                    'size' => 128,
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Token API Secret'),
                    'name' => 'OZON_API_TOKEN_SECRET',
                    'size' => 128,
                )
            )
        );

        $helper = new HelperForm();
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = $this->context->language->id;
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;
        $helper->toolbar_scroll = true;
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = array(
            'save' => array(
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                    '&token='.Tools::getAdminTokenLite('AdminModules'),
            ),
            'back' => array(
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );
        $helper->tpl_vars = array(
            'uri' => $this->getPathUri(),
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'ps_module_form_dir' => _PS_MODULE_DIR_.$this->name.'/views/templates/admin/_configure/helpers/form/'
        );

        return $helper->generateForm($fields_form);
    }

    /**
     * @return array
     */
    private function getConfigFieldsValues()
    {
        $fields = array();

        $fields['OZON_ACTIVE'] = html_entity_decode(Tools::getValue('OZON_ACTIVE', Configuration::get('OZON_ACTIVE')));
        $fields['OZON_API_URL'] = html_entity_decode(
            Tools::getValue('OZON_API_URL', Configuration::get('OZON_API_URL'))
        );
        $fields['OZON_WEBAPP_TOKEN'] = html_entity_decode(
            Tools::getValue('OZON_WEBAPP_TOKEN', Configuration::get('OZON_WEBAPP_TOKEN'))
        );
        $fields['OZON_API_TOKEN'] = html_entity_decode(
            Tools::getValue('OZON_API_TOKEN', Configuration::get('OZON_API_TOKEN'))
        );
        $fields['OZON_API_TOKEN_SECRET'] = html_entity_decode(
            Tools::getValue('OZON_API_TOKEN_SECRET', Configuration::get('OZON_API_TOKEN_SECRET'))
        );

        $fields['submitKey'] = 1;

        return $fields;
    }

    //=====================================
    //  TOOLS FUNCTIONS
    //=====================================
    /**
     * Get color associated to the risk constant.
     * @param string $order_risk
     * @return string
     */
    public function getColorOrderRisk($order_risk = '')
    {
        $default = '#CCCCCC';
        $colors = array(
            'low' => '#0000FF',
            'medium' => '#6D5FAA',
            'high' => '#FF0000',
        );

        if (isset($colors[$order_risk])) {
            return $colors[$order_risk];
        }

        return $default;
    }

    /**
     * Send specific order to API and save results.
     * @param $order
     * @param bool|false $return_data
     * @return array|bool
     */
    public function sendOrderToAPI($order, $return_data = false)
    {
        // Check if module is active.
        if (!$this->moduleIsActive()) {
            return false;
        }

        // Check API config data.
        if (!Configuration::get('OZON_API_URL') || !Configuration::get('OZON_API_TOKEN')) {
            NQLogger::addLog('Error : invalid API url or token.', -1);
            if ($return_data) {
                return array(
                    'success' => false,
                    'message' => 'Error : invalid API url or token.'
                );
            }
            return false;
        }

        if (!is_object($order)) {
            $order = new Order((int)$order);
        }

        /*
         * Check all order data before API call.
         */
        if (!Validate::isLoadedObject($order)) {
            NQLogger::addLog('Error orders/check/ : Invalid order.', -1);
            if ($return_data) {
                return array(
                    'success' => false,
                    'message' => 'Error orders/check/ : Invalid order.',
                );
            }
            return false;
        }
        $customer = new Customer((int)$order->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            NQLogger::addLog('Error orders/check/ : Unknown customer ['.$order->id_customer.'].', $order->id);
            if ($return_data) {
                return array(
                    'success' => false,
                    'message' => 'Error orders/check/ : Unknown customer ['.$order->id_customer.'].',
                );
            }
            return false;
        }
        $address_shipping = new Address((int)$order->id_address_delivery);
        if (!Validate::isLoadedObject($order)) {
            NQLogger::addLog('Error orders/check/ : Unknown shipping address ['.$order->id_address_delivery.'].
            ', $order->id);
            if ($return_data) {
                return array(
                    'success' => false,
                    'message' => 'Error orders/check/ : Unknown shipping address ['.$order->id_address_delivery.'].',
                );
            }
            return false;
        }
        $address_billing = new Address((int)$order->id_address_invoice);
        if (!Validate::isLoadedObject($order)) {
            NQLogger::addLog('Error orders/check/ : Unknown billing address ['.$order->id_address_invoice.'].
            ', $order->id);
            if ($return_data) {
                return array(
                    'success' => false,
                    'message' => 'Error orders/check/ : Unknown billing address ['.$order->id_address_invoice.'].',
                );
            }
            return false;
        }
        $currency = new Currency((int)$order->id_currency);
        if (!Validate::isLoadedObject($order)) {
            NQLogger::addLog('Error orders/check/ : Unknown order currency ['.$order->id_currency.'].', $order->id);
            if ($return_data) {
                return array(
                    'success' => false,
                    'message' => 'Error orders/check/ : Unknown order currency ['.$order->id_currency.'].',
                );
            }
            return false;
        }
        $ozon_uuid = OzonUuidIndex::getIndex($order->id_cart);
        if (empty($ozon_uuid)) {
            NQLogger::addLog('Warning orders/check/ : Empty ozon_uuid', $order->id);
            if ($return_data) {
                return array(
                    'success' => false,
                    'message' => 'Warning orders/check/ : Empty ozon_uuid',
                );
            }
            return false;
        }

        /*
         * Format data to send api call.
         */
        $tab_sku = array();
        $products = $order->getProducts();
        if (is_array($products)) {
            foreach ($products as $product) {
                if (!empty($product['reference'])) {
                    $tab_sku[] = $product['reference'];
                }
            }
        }

        $data = array(
            'ozon_uuid' => $ozon_uuid,
            'ip' => Tools::getRemoteAddr(),
            'email' => $customer->email,
            'order_id' => $order->id,
            'order_amount' => $order->total_paid,
            'order_currency' => $currency->iso_code,
            'list_sku' => count($tab_sku) ? implode(',', $tab_sku) : '',
            'billing_address' => $address_billing->address1.' '.$address_billing->address2,
            'billing_city' => $address_billing->city,
            'billing_postal' => $address_billing->postcode,
            'billing_country' => Country::getIsoById($address_billing->id_country),
            'shipping_address' => $address_shipping->address1.' '.$address_shipping->address2,
            'shipping_city' => $address_shipping->city,
            'shipping_postal' => $address_shipping->postcode,
            'shipping_country' => Country::getIsoById($address_shipping->id_country),
        );

        /*
         * Send api call.
         */
        try {
            $clientAPI = new OzonClientApi();
            $result = $clientAPI->checkOrder($data);

            if ($result === false) {

                OzonOrderindex::saveIndex(
                    (int)$order->id,
                    0,
                    0,
                    0,
                    '',
                    0,
                    '',
                    0,
                    0,
                    'Unknown error'
                );

                NQLogger::addLog('Error orders/check/ : Empty response.', $order->id);
                if ($return_data) {
                    return array(
                        'success' => false,
                        'message' => 'Error orders/check/ : Empty response.',
                    );
                }
            } elseif (!$result['success']) {

                OzonOrderindex::saveIndex(
                    (int)$order->id,
                    0,
                    0,
                    0,
                    '',
                    0,
                    '',
                    0,
                    0,
                    $result['message']
                );

                NQLogger::addLog(
                    'Error orders/check/ : '.(!empty($result['message']) ? $result['message'] : ''),
                    $order->id
                );
                if ($return_data) {
                    return array(
                        'success' => false,
                        'message' => 'Error orders/check/ : '.(!empty($result['message']) ? $result['message'] : ''),
                    );
                }
            } else {

                OzonOrderindex::saveIndex(
                    (int)$order->id,
                    1,
                    $result['ozon_order_id'],
                    $result['score'],
                    $result['order_risk'],
                    $result['proxy_score'],
                    $result['ip_user_type'],
                    ($result['high_risk_email'] ? 1 : 0),
                    ($result['proxy_anonymous'] ? 1 : 0),
                    (!empty($result['message']) ? $result['message'] : ''),
                    $result['tags']
                );

            }

        } catch (Exception $e) {
            NQLogger::addLog('Error orders/check/ (exception occurred) '.$e->getMessage(), $order->id);
            if ($return_data) {
                return array(
                    'success' => false,
                    'message' => 'Error orders/check/ (exception occurred) '.$e->getMessage(),
                );
            }
            return false;
        }

        if ($return_data) {
            return array(
                'success' => true,
                'message' => '',
            );
        }
        return true;
    }
}
